package com.steerinfo.dil.service.impl;

import com.alibaba.fastjson.JSON;
import com.steerinfo.dil.mapper.AmsNotUploadedEasMapper;
import com.steerinfo.dil.model.AmsNotUploadedEas;
import com.steerinfo.dil.wsdl.LoginUtil;
import com.steerinfo.dil.wsdl.ServiceInfo;
import com.steerinfo.dil.wsdl.WSContext;
import com.steerinfo.dil.wsdl.WSUpDaxingMeteringSystemFacadeSrvProxyServiceLocator;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @ author    :TXF
 * @ time      :2022/2/10 10:07
 */

@Service
public class KDWebserviceUtil {

    @Autowired
    private AmsNotUploadedEasMapper amsNotUploadedEasMapper;

    /**
     *
     * 推送数据给金蝶接口
     * @Author TXF
     * @Date 2022/2/10 17:16
     * @param map n个数据
     * @return Integer   1 成功  0 失败
     **/
    public int sendMesToKD(Map<String, Object> map){
        try {
            String xml = asXmlData(map);
            System.out.println(xml);
            ServiceInfo service = new ServiceInfo("user", "dgjt@kingdee", "eas", "A004", "L2", 1);
            service.setServername("172.16.62.243" , "8888");
            WSContext wsContext = LoginUtil.doEASLogin(service);
            String sessionId = wsContext.getSessionId();
            if(null == sessionId) {
                addNotUploadedEASResult(map);
                System.out.println("[error]: 登陆有问题");
                return 0;
            }
            WSUpDaxingMeteringSystemFacadeSrvProxyServiceLocator locator = new WSUpDaxingMeteringSystemFacadeSrvProxyServiceLocator();
            String res = locator.getWSUpDaxingMeteringSystemFacade().saveWeighBill(xml);
            Map returnMap = JSON.parseObject(res, Map.class);
            if("S02".equals(String.valueOf(returnMap.get("Status")))){
                return 1;
            }else {
                addNotUploadedEASResult(map);
                System.out.println((map.get("orderNumber")+ "数据推送金蝶失败："+ returnMap.get("MessageText")));
                return 0;
            }
        } catch (Exception e) {
            addNotUploadedEASResult(map);
            System.out.println(map.get("orderNumber") + "数据推送金蝶失败：" + e.getMessage());
            return 0;
        }
    }


    /**
     * 创建计量数据 金蝶xml格式字符串
     * @Author TXF
     * @Date 2022/2/10 17:32
     * @param map
     * @return
     **/
    public String asXmlData(Map<String, Object> map){
        // 创建一个Document实例
        Document doc = DocumentHelper.createDocument();
        // 添加根节点
        Element requests = doc.addElement("requests");
        //添加第子节点
        Element request = requests.addElement("request");
        request.addElement("TypeID").addText("PUR");               //采购订单类型
        request.addElement("FNumber").addText((String) map.get("resultPoundNo"));              //磅单号
        request.addElement("FBizDate").addText((String) map.get("resultTareWeightTime"));             // 净重时间
        request.addElement("Carnumber").addText((String) map.get("capacityNumber"));            //车牌号
        request.addElement("FTransportOrgID").addText("");                          //运输单位
        request.addElement("FGrossScaleID").addText((String) map.get("resultCrossCalculateNumber"));        //毛重汽车衡
        request.addElement("FTareScaleID").addText((String) map.get("resultTareCalculateNumber"));         //皮重汽车衡
        request.addElement("FGrossWeight").addText((String) map.get("resultCrossWeight"));         //毛重
        request.addElement("FTareWeight").addText((String) map.get("resultTareWeight"));          //皮重
        request.addElement("FGrossPoundTime").addText((String) map.get("resultCrossWeightTime"));      //毛重时间
        request.addElement("FTarePoundTime").addText((String) map.get("resultTareWeightTime"));       //皮重时间
        request.addElement("FSourceBillID").addText((String) map.get("No"));        //采购订单号
        request.addElement("FSourceBillEntryID").addText((String) map.get("easEntryId"));   //分录ID
        request.addElement("FNetWeight").addText((String) map.get("resultNetWeight"));           //净重
        request.addElement("FIncidental").addText("0");         //扣杂
        request.addElement("FLastNetWeight").addText((String) map.get("resultNetWeight"));     //最后净重
        request.addElement("FDescription").addText("");       //备注
        return doc.asXML();
    }

    /**
     * 如果回传金蝶失败则将数据保存到数据库
     * @Author TXF
     * @Date 2022/2/14 16:35
     * @param map
     * @return
     **/
    public int addNotUploadedEASResult(Map<String, Object> map) {
        AmsNotUploadedEas amsNotUploadedEas = new AmsNotUploadedEas();
        //查询该订单是否已存在
        String orderId = (String) map.get("orderId");
        String isExistsOrderId = amsNotUploadedEasMapper.getIsExistsOrderId(orderId);
        if(isExistsOrderId == null){
            return 0;
        }
        amsNotUploadedEas.setId(orderId);
        amsNotUploadedEas.setNo((String) map.get("No"));
        amsNotUploadedEas.setResultPoundNo((String) map.get("resultPoundNo"));
        amsNotUploadedEas.setCapacityNumber((String) map.get("capacityNumber"));
        amsNotUploadedEas.setResultCrossCalculateNumber((String) map.get("resultCrossCalculateNumber"));
        amsNotUploadedEas.setResultTareCalculateNumber((String) map.get("resultTareCalculateNumber"));
        amsNotUploadedEas.setResultCrossWeight((String) map.get("resultCrossWeight"));
        amsNotUploadedEas.setResultTareWeight((String) map.get("resultTareWeight"));
        amsNotUploadedEas.setResultCrossWeightTime((String) map.get("resultCrossWeightTime"));
        amsNotUploadedEas.setResultTareWeightTime((String) map.get("resultTareWeightTime"));
        amsNotUploadedEas.setResultNetWeight((String) map.get("resultNetWeight"));
        amsNotUploadedEas.setEasEntryId((String) map.get("easEntryId"));
        return amsNotUploadedEasMapper.insertSelective(amsNotUploadedEas);
    }

    /**
     * 定时访问未上传金蝶数据 重复抛
     * @Author TXF
     * @Date 2022/2/14 17:01
     * @param
     * @return
     **/
    @Scheduled(fixedRate = 1000 * 60 * 10)
    public void timingPushMesToKD(){
        List<Map<String, Object>> allNotUploadedEasMes = amsNotUploadedEasMapper.getAllNotUploadedEasMes();
        if(allNotUploadedEasMes != null && allNotUploadedEasMes.size() != 0){
            for (Map<String, Object> map : allNotUploadedEasMes) {
                int i = sendMesToKD(map);
                String orderId = (String) map.get("orderId");
                if (i == 1){
                    //如果添加成功则往数据库中删除此数据
                    amsNotUploadedEasMapper.deleteByPrimaryKey(orderId);
                    System.out.println(orderId + map.get("capacityNumber") + "补传成功");
                }else {
                    System.out.println(orderId + map.get("capacityNumber") + "补传失败");
                }
            }
        }
    }

}






/**
 * 封装的访问webservice的工具类:（有局限性，数据量小的xml报文可用，较大则会出现stackoverflow的异常）
 *
 * @param reqXML
 *            请求的xml
 * @param endPoint
 *            webservice的路径除去?wsdl:例如：http://1.202.235.66:7002/webService/PartsService（没有?wsdl）
 * @param namespace
 *            名称空间
 * @param method
 *            方法名称
 * @return	返回报文结果(xml)
 */
//    public static String loadWebService(String reqXML, String endPoint, String namespace, String method) {
//        String responseXml = "";
//        RPCServiceClient serviceClient = null;
//        try {
//            /* 调用理赔系统的接口 */
//            String nameSpace = namespace;
//            QName qname = new QName(nameSpace, method);
//            serviceClient = new RPCServiceClient();
//            Options options = serviceClient.getOptions();
//            EndpointReference targetEPR = new EndpointReference(endPoint);
//            options.setTo(targetEPR);
//            options.setTimeOutInMilliSeconds(30000);
//            Object[] rtnObjs = serviceClient.invokeBlocking(qname, new Object[] { reqXML },
//                    new Class[] { String.class });
//            responseXml = rtnObjs[0].toString();
//        } catch (Exception e) {
//            e.toString();
//        }
//        return responseXml;
//    }

/**
 * axis方式访问webservice
 *
 * @param
 *            请求的xml的报文
 * @param
 *            webservice的wsdl地址,例如：http://1.202.235.66:7002/webService/PartsService?wsdl
 * @param
 *            调用方法的命名空间
 * @param
 *            调用的方法名
 * @return 返回调用的报文(xml)
 *
 */
//    public static String invokRemoveWSDLByAxis(String reqXML, String endPoint, String namespace, String method) {
//        String responseXML = "";
//        Service service = new Service();
//        try {
//            Call call = (Call) service.createCall();
//            call.setTargetEndpointAddress(endPoint);
//            call.setOperationName(new javax.xml.namespace.QName(namespace, method));
//            call.addParameter("arg0", XMLType.XSD_STRING, ParameterMode.IN);
//            call.setReturnType(XMLType.XSD_STRING);
//            responseXML = (String) call.invoke(new Object[] { reqXML });
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return responseXML;
//    }
